/**
 * HiraganaHandler.java
 *
 * Copyright (c) 1997- Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.henya;

import java.awt.*;

/**
 * Handle moving hiragana letters.
 *
 * @author cs8k-cyu@asahi-net.or.jp
 * @version 0.92
 */
public class HiraganaHandler {
  private static Color bl = new Color(0,0,31);
  private static Color wh = new Color(255,255,255);

  private Graphics grf;
  private Dimension wd;

  private final static int HLMAX = 256;
  private HnLine[] hl = new HnLine[HLMAX];
  private int[] hp = new int[61];

  public HiraganaHandler(Graphics g, Dimension d, String lw) {
    grf = g; wd = d;
    int i, j;
    Integer ti;
    for ( i=0 ; i<HLMAX ; i++ ) hl[i] = new HnLine(grf);
    try {
      ti = new Integer(lw);
      hl[0].setLineWidth(ti.intValue());
    } catch ( NumberFormatException e ) {
      hl[0].setLineWidth(3);
    }
    j = 0;
    for ( i=0 ; i<61 ; i++ ) {
      hp[i] = j;
      while ( hirapoint[j] != -32768 ) j++;
      j++;
    }
  }

  public void setStr(int[] chr, int[] atr,int zoom, int wx, int rw) {
    int i, j, k, l;
    int c;
    int tzm, stx, tstx, sty, tsty;
    for ( i=0 ; i<HLMAX ; i++ ) hl[i].cou = -1;
    j = 0;
    stx = (wd.width-(chr.length-1)*wx-zoom)/2;
    sty = (wd.height-zoom)/2;
    for ( i=0 ; i<chr.length ; i++ ) {
      c = chr[i];
      tzm = zoom; tstx = stx; tsty = sty;
      switch ( atr[i] ) {
      case 0:
        break;
      case 1:
        tzm = zoom/2; tstx = stx+tzm/2; tsty = sty+tzm;
        break;
      case 2:
      case 3:
        if ( atr[i] == 2 ) {
          k = hp[46];
        } else {
          k = hp[47];
        }
        while ( true ) {
          l = 0; hl[j].cou = 0; hl[j].rw = rw;
          while ( hirapoint[k] > -32760 ) {
            hl[j].xa[l] = tstx+hirapoint[k]*tzm/256; k++;
            hl[j].ya[l] = tsty+hirapoint[k]*tzm/256; k++;
            l++;
          }
          hl[j].xa[l] = -32760;
          if ( hirapoint[k] ==- 32768 ) break;
          k++; j++; if ( j >= HLMAX ) return;
        }
        j++; if ( j >= HLMAX ) return;
        break;
      }
      if ( c >= 0 ) {
        k = hp[c];
        while (true) {
          l = 0; hl[j].cou = 0; hl[j].rw = rw;
          while ( hirapoint[k] > -32760 ) {
            hl[j].xa[l] = tstx+hirapoint[k]*tzm/256; k++;
            hl[j].ya[l] = tsty+hirapoint[k]*tzm/256; k++;
            l++;
          }
          hl[j].xa[l] = -32760;
          if ( hirapoint[k] == -32768 ) break;
          k++; j++;
        }
        j++;
      }
      stx+=wx;
    }
  }

  public void move() {
    grf.setColor(bl);
    grf.fillRect(0,0,wd.width,wd.height);
    grf.setColor(wh);
    for ( int i=0 ; i<HLMAX ; i++ ) hl[i].move();
  }

private static final int[] hirapoint = {
54,130,220,115,-32768,

32,64,192,58,-32760,100,16,128,230,-32760,192,128,50,250,30,130,
220,130,192,250,-32768,
32,64,40,240,-32760,230,32,230,240,-32768,
90,20,170,64,-32760,64,100,200,100,115,250,-32768,
90,10,170,64,-32760,64,90,200,70,50,250,130,180,140,250,250,250,-32768,
70,50,150,50,-32760,120,10,84,250,32,130,230,128,160,250,-32760,
190,50,250,128,-32768,

64,128,160,100,138,250,-32760,128,40,64,250,-32760,192,64,240,128,-32768,
64,64,192,32,-32760,66,128,200,96,-32760,128,10,140,180,-32760,64,192,128,250,-32768,
150,10,100,128,150,250,-32768,
50,32,50,240,-32760,118,90,230,90,-32760,180,20,140,250,-32768,
64,50,192,30,-32760,64,192,200,250,-32768,

64,90,192,60,-32760,110,32,130,140,-32760,64,192,130,250,-32768,
90,30,92,220,160,250,220,210,-32768,
60,70,220,70,-32760,128,20,128,190,100,140,145,140,120,250,-32768,
40,70,200,70,-32760,150,20,150,128,-32760,70,20,80,222,220,230,-32768,
80,40,192,30,50,128,192,120,128,200,192,250,-32768,
50,90,150,64,-32760,100,20,60,240,-32760,110,140,210,120,-32760,
100,200,230,250,-32768,
70,64,200,50,-32760,128,20,90,140,200,140,120,250,-32768,
50,100,220,70,130,210,-32768,
60,64,200,40,128,140,200,240,-32768,
50,40,128,128,-32760,192,30,64,230,230,230,-32768,
32,64,130,60,-32760,100,10,32,190,-32760,150,64,210,128,-32760,
128,100,120,250,80,192,200,192,-32768,
60,32,60,192,-32760,100,70,210,60,-32760,110,180,220,250,-32768,
64,50,110,230,-32760,200,64,64,240,50,100,200,100,150,250,
130,190,210,240,-32768,
70,40,70,240,-32760,32,100,110,64,40,210,200,64,150,250,128,192,220,230,-32768,
128,64,90,240,40,64,220,64,140,250,-32768,
60,40,60,250,-32760,100,90,230,90,-32760,140,32,140,250,100,180,220,240,-32768,
10,70,70,40,64,230,140,250,160,70,240,128,-32768,
100,20,140,100,-32760,110,110,130,240,-32760,64,140,32,240,-32760,
180,130,220,210,-32768,
20,150,100,100,250,192,-32768,
64,30,64,250,-32760,110,40,210,40,-32760,110,128,220,128,-32760,170,40,150,250,115,192,210,240,-32768,
60,50,180,50,-32760,64,128,192,128,-32760,125,20,120,250,80,190,190,240,-32768,
50,60,115,60,70,250,40,140,230,140,-32760,170,110,160,230,-32768,
32,64,110,60,-32760,60,32,64,170,30,128,60,120,70,240,220,240,-32760,
150,64,210,128,-32768,
64,64,128,220,-32760,192,64,64,250,20,100,230,100,180,250,-32768,
50,64,140,64,-32760,50,135,150,135,-32760,100,20,100,250,210,212,-32768,
40,100,190,80,130,150,-32760,140,30,128,90,-32760,64,40,90,250,-32768,
60,64,60,230,130,70,240,150,100,240,-32760,130,20,130,250,-32768,
120,70,170,70,-32760,120,30,120,250,64,180,210,250,-32768,
90,40,180,90,-32760,120,90,100,150,192,150,128,250,-32768,
70,32,70,150,-32760,150,32,150,250,-32768,
92,60,190,50,50,140,192,128,120,250,115,192,140,210,-32768,
64,40,64,240,-32760,40,90,90,64,40,200,170,100,150,250,240,192,-32768,
64,64,170,50,64,150,170,128,128,250,-32768,
70,40,70,250,-32760,40,80,100,60,30,200,170,80,220,160,200,250,-32768,
128,20,30,220,128,128,140,240,240,240,-32768,
220,20,250,64,-32760,240,10,270,50,-32768,
220,10,250,10,250,40,220,40,220,10,-32768,
70,40,160,40,-32760,128,20,70,128,130,100,130,190,-32760,150,110,70,240,192,230,-32768,

160,10,30,100,10,200,120,250,200,180,250,100,160,10,-32768,
140,10,120,250,-32768,
50,80,150,10,220,80,40,250,200,250,-32768,
60,40,230,30,200,110,100,128,210,150,235,250,50,250,-32768,
240,150,30,150,150,20,140,256,-32768,
240,10,40,10,30,120,200,140,210,220,20,250,-32768,
160,10,40,150,30,250,200,220,240,130,40,150,-32768,
50,20,220,10,130,250,-32768,
200,50,130,20,50,60,240,130,220,250,30,200,200,50,-32768,
230,60,200,10,100,10,40,100,230,60,100,250,-32768,

64,16,0,240,64,245,96,24,64,16,-32760,150,32,192,220,250,210,200,16,150,32,-32768,
220,16,40,128,64,210,122,135,115,250,156,250,150,110,240,40,220,16,-32768,
-99999
};

}
