/**
 * Messages.java
 *
 * Copyright (c) 1997- Kenta Cho. All rights reserved.
 */
package jp.gr.java_conf.abagames.henya;

/**
 * Parse romaji messages.
 *
 * @author cs8k-cyu@asahi-net.or.jp
 * @version 0.92
 */
public class Messages {
  private final static int MESMAX = 256;
  private final static int STRMAX = 256;
  public static int lastnum = 0;

  public int[][] chr = new int[MESMAX][];
  public int[][] atr = new int[MESMAX][];
  public int[] pl = new int[MESMAX];
  public int[] zm = new int[MESMAX];

  private final String special = " -";
  private final String vowel = "aiueo";                //0
  private final String consonant_with_y = "ksthmr";    //1
  private final String consonant_with_n = "n";         //2
  private final String consonant = "yw";               //3
  private final String voiced_with_y = "gzb";          //4
  private final String voiced = "d";                   //5
  private final String halfvoiced = "p";               //6
  private final String consonant_with_v = "f";         //7
  private final String small_prefix = "x";             //8

  private final int[] cases = {
    0,4,-1,5,0,7,4,1,0,-1,1,-1,1,2,0,6,-1,1,1,1,0,-1,3,8,3,4
  };

  private final String consonants = "ksthmrnywgzbdp";

  private final int[] consonants_table = {
    6,11,16,26,31,39,21,36,44,6,11,26,16,26
  };

  private final int[] sml_lt_table = {
    36,36,37,4,38
  };

  private void parseRomaji(String str, int msgIdx) {
    int rslIdx = 0;
    str = str.toLowerCase();
    int idx = 0;
    int[] tc = new int[STRMAX];
    int[] ta = new int[STRMAX];
    while ( idx < str.length() && rslIdx < STRMAX-10 ) {
      int ch = (int)str.charAt(idx);
      int spi;
      int cs = -1;
      // ' ' '-'
      if ( (spi = special.indexOf(ch)) != -1 ) {
        tc[rslIdx] = -1+spi;
        rslIdx++;
      // 0-9
      } if ( ch>='0' && ch<='9' ) {
        tc[rslIdx] = 49 + ch-'0';
        rslIdx++;
      } else {
        try {
          cs = cases[ch-'a'];
        } catch ( ArrayIndexOutOfBoundsException e ) {}
        // small 'tu'
        if ( cs>=1 && cs<=7 && ch != 'n' ) {
          idx++; if ( idx >= str.length() ) break;
          if ( ch == (int)str.charAt(idx) ) {
            tc[rslIdx] = 18;
            ta[rslIdx] = 1;
            rslIdx++;
          } else {
            idx--;
          }
        }
      }
      switch ( cs ) {
      // a i u e o
      case 0:
        tc[rslIdx] = 1 + vowel.indexOf(ch);
        rslIdx++;
        break;
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
        idx++; if ( idx >= str.length() ) break;
        // nn
        if ( cs == 2 ) {
          if ( str.charAt(idx) == 'n' ) {
            tc[rslIdx] = 45;
            rslIdx++;
            break;
          }
        }
        // ?ya ?yu ?yo
        if ( cs != 3 && cs != 5 && str.charAt(idx) == 'y' ) {
          idx++; if ( idx >= str.length() ) break;
          int vwi;
          if ( (vwi = vowel.indexOf(str.charAt(idx))) != -1 ) {
            int cwi;
            if ( (cwi = consonants.indexOf(ch)) != -1 ) {
              tc[rslIdx] = consonants_table[cwi]+1;
              switch ( cs ) {
              case 4:
              case 5:
                ta[rslIdx] = 2;
                break;
              case 6:
                ta[rslIdx] = 3;
                break;
              }
              rslIdx++;
              tc[rslIdx] = sml_lt_table[vwi];
              ta[rslIdx] = 1;
              rslIdx++;
            }
          }
        // ?a ?i ?u ?e ?o
        } else {
          int vwi;
          if ( (vwi = vowel.indexOf(str.charAt(idx))) != -1 ) {
            int cwi;
            if ( (cwi = consonants.indexOf(ch)) != -1 ) {
              if ( ch == 'y' ) {
                tc[rslIdx] = consonants_table[cwi] + vwi/2;
              } else if ( ch == 'w' ) {
                if ( vwi == 4 ) {
                  tc[rslIdx] = 48;
                } else {
                  tc[rslIdx] = consonants_table[cwi];
                }
              } else {
                tc[rslIdx] = consonants_table[cwi] + vwi;
              }
              switch ( cs ) {
              case 4:
              case 5:
                ta[rslIdx] = 2;
                break;
              case 6:
                ta[rslIdx] = 3;
                break;
              }
              rslIdx++;
            }
          }
        }
        break;
      // fa fi fe fo
      case 7:
        idx++; if ( idx >= str.length() ) break;
        int vwi;
        if ( (vwi = vowel.indexOf(str.charAt(idx))) != -1 ) {
          tc[rslIdx] = 27;
          rslIdx++;
          tc[rslIdx] = 1 + vwi;
          ta[rslIdx] = 1;
          rslIdx++;
        }
        break;
      // small letter prefix
      case 8:
        ta[rslIdx] = 1;
        break;
      }
      idx++;
    }
    chr[msgIdx] = new int[rslIdx];
    System.arraycopy(tc, 0, chr[msgIdx], 0, rslIdx);
    atr[msgIdx] = new int[rslIdx];
    System.arraycopy(ta, 0, atr[msgIdx], 0, rslIdx);
  }

  public Messages(String messt) {
    Integer ti;
    int i = 0, sp = 0, ep;

    if ( messt == null ) messt = "youkoso///kokohe///";

    while ( (ep=messt.indexOf('/',sp)) != -1 ) {
      parseRomaji(messt.substring(sp,ep), i);

      sp = ep+1; ep = messt.indexOf('/',sp);
      try {
        ti = new Integer(messt.substring(sp,ep));
        zm[i] = ti.intValue();
      } catch (NumberFormatException e) {
        zm[i] = 64;
      }
      sp = ep+1; ep = messt.indexOf('/',sp);
      try {
        ti = new Integer(messt.substring(sp,ep));
        pl[i] = ti.intValue();
      } catch (NumberFormatException e) {
        pl[i] = 80;
      }
      sp = ep+1;
      i++;
    }
    lastnum = i;
  }
}
